
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "antObject.h"
#include "antTemplate.h"

#include <string.h>

namespace ant
{

Object::Object ()
  : m_p1 (), m_p2 (), m_id (0),
    m_fmt_x ("$X"), m_fmt_y ("$Y"), m_fmt ("$D"),
    m_style (STY_ruler), m_outline (OL_diag),
    m_snap (true), m_angle_constraint (ant::Object::Global)
{
  //  .. nothing yet ..
}

Object::Object (const db::DPoint &p1, const db::DPoint &p2, int id, const std::string &fmt_x, const std::string &fmt_y, const std::string &fmt, style_type style, outline_type outline, bool snap, ant::Object::angle_constraint_type angle_constraint)
  : m_p1 (p1), m_p2 (p2), m_id (id),
    m_fmt_x (fmt_x), m_fmt_y (fmt_y), m_fmt (fmt),
    m_style (style), m_outline (outline),
    m_snap (snap), m_angle_constraint (angle_constraint)
{
  //  .. nothing else ..
}

Object::Object (const db::DPoint &p1, const db::DPoint &p2, int id, const ant::Template &t)
  : m_p1 (p1), m_p2 (p2), m_id (id),
    m_fmt_x (t.fmt_x ()), m_fmt_y (t.fmt_y ()), m_fmt (t.fmt ()),
    m_style (t.style ()), m_outline (t.outline ()),
    m_snap (t.snap ()), m_angle_constraint (t.angle_constraint ())
{
  //  .. nothing else ..
}

Object::Object (const ant::Object &d)
  : m_p1 (d.m_p1), m_p2 (d.m_p2), m_id (d.m_id),
    m_fmt_x (d.m_fmt_x), m_fmt_y (d.m_fmt_y), m_fmt (d.m_fmt),
    m_style (d.m_style), m_outline (d.m_outline),
    m_snap (d.m_snap), m_angle_constraint (d.m_angle_constraint)
{
  //  .. nothing else ..
}

Object &
Object::operator= (const ant::Object &d)
{
  if (this != &d) {
    m_p1 = d.m_p1;
    m_p2 = d.m_p2;
    m_id = d.m_id;
    m_fmt_x = d.m_fmt_x;
    m_fmt_y = d.m_fmt_y;
    m_fmt = d.m_fmt;
    m_style = d.m_style;
    m_outline = d.m_outline;
    m_snap = d.m_snap;
    m_angle_constraint = d.m_angle_constraint;
  }
  return *this;
}

bool 
Object::operator< (const ant::Object &b) const
{
  if (m_id != b.m_id) {
    return m_id < b.m_id;
  }
  if (m_p1 != b.m_p1) {
    return m_p1 < b.m_p1;
  }
  if (m_p2 != b.m_p2) {
    return m_p2 < b.m_p2;
  }
  if (m_fmt_x != b.m_fmt_x) {
    return m_fmt_x < b.m_fmt_x;
  }
  if (m_fmt_y != b.m_fmt_y) {
    return m_fmt_y < b.m_fmt_y;
  }
  if (m_fmt != b.m_fmt) {
    return m_fmt < b.m_fmt;
  }
  if (m_style != b.m_style) {
    return m_style < b.m_style;
  }
  if (m_outline != b.m_outline) {
    return m_outline < b.m_outline;
  }
  if (m_snap != b.m_snap) {
    return m_snap < b.m_snap;
  }
  if (m_angle_constraint != b.m_angle_constraint) {
    return m_angle_constraint < b.m_angle_constraint;
  }
  return false;
}

bool 
Object::equals (const db::DUserObjectBase *d) const
{
  const ant::Object *ruler = dynamic_cast<const ant::Object *> (d);
  if (ruler) {
    return m_id == ruler->m_id && m_p1 == ruler->m_p1 && m_p2 == ruler->m_p2 &&
           m_fmt_x == ruler->m_fmt_x && m_fmt_y == ruler->m_fmt_y && m_fmt == ruler->m_fmt &&
           m_style == ruler->m_style && m_outline == ruler->m_outline &&
           m_snap == ruler->m_snap && m_angle_constraint == ruler->m_angle_constraint;
  } else {
    return false;
  }
}

bool 
Object::operator== (const ant::Object &d) const
{
  return m_p1 == d.m_p1 && m_p2 == d.m_p2 && m_id == d.m_id && 
         m_fmt_x == d.m_fmt_x && m_fmt_y == d.m_fmt_y && m_fmt == d.m_fmt &&
         m_style == d.m_style && m_outline == d.m_outline && 
         m_snap == d.m_snap && m_angle_constraint == d.m_angle_constraint;
}

bool 
Object::less (const db::DUserObjectBase *d) const
{
  const ant::Object *ruler = dynamic_cast<const ant::Object *> (d);
  tl_assert (ruler != 0);

  if (m_id != ruler->m_id) {
    return m_id < ruler->m_id;
  }
  if (m_p1 != ruler->m_p1) {
    return m_p1 < ruler->m_p1;
  }
  if (m_p2 != ruler->m_p2) {
    return m_p2 < ruler->m_p2;
  }
  if (m_fmt_x != ruler->m_fmt_x) {
    return m_fmt_x < ruler->m_fmt_x;
  }
  if (m_fmt_y != ruler->m_fmt_y) {
    return m_fmt_y < ruler->m_fmt_y;
  }
  if (m_fmt != ruler->m_fmt) {
    return m_fmt < ruler->m_fmt;
  }
  if (m_style != ruler->m_style) {
    return m_style < ruler->m_style;
  }
  if (m_outline != ruler->m_outline) {
    return m_outline < ruler->m_outline;
  }
  if (m_snap != ruler->m_snap) {
    return m_snap < ruler->m_snap;
  }
  if (m_angle_constraint != ruler->m_angle_constraint) {
    return m_angle_constraint < ruler->m_angle_constraint;
  }
  return false;
}

unsigned int 
Object::class_id () const
{
  static unsigned int cid = db::get_unique_user_object_class_id ();
  return cid;
}

db::DUserObjectBase *
Object::clone () const
{
  return new ant::Object (*this);
}

db::DBox 
Object::box () const
{
  return db::DBox (m_p1, m_p2);
}

std::string 
Object::formatted (const std::string &fmt) const
{
  std::string r;
  r.reserve (fmt.size ());

  const char *f = fmt.c_str ();
  while (*f) {

    while (*f && ((*f != '$' && *f != '\\') || !f[1])) {
      r += *f++;
    }

    if (*f == '\\') {

      char code = *++f;
      if (code == 'n') {
        r += '\n';
      } else {
        r += code;
      }
      ++f;

    } else if (*f == '$') {

      char code = *++f;

      if (! code) {
        r += '$';
      } else if (code == '$') {
        ++f;
        r += '$';
      } else {
        
        ++f;

        double v = 0.0;
        double dx = (p1 ().x () - p2 ().x ());
        double dy = (p1 ().y () - p2 ().y ());

        //  avoid "almost 0" outputs
        if (fabs (dx) < 1e-5 /*micron*/) {
          dx = 0;
        }
        if (fabs (dy) < 1e-5 /*micron*/) {
          dy = 0;
        }

        switch (code) {
        case 'l':
        case 'L':
          v = fabs (dx) + fabs (dy);
          if (code == 'l') {
            v *= 1e3;
          }
          break;
        case 'd':
        case 'D':
          v = sqrt (dx * dx + dy * dy);
          if (code == 'd') {
            v *= 1e3;
          }
          break;
        case 'u':
        case 'U':
          v = p1 ().x();
          if (code == 'u') {
            v *= 1e3;
          }
          break;
        case 'v':
        case 'V':
          v = p1 ().y();
          if (code == 'v') {
            v *= 1e3;
          }
          break;
        case 'p':
        case 'P':
          v = p2 ().x();
          if (code == 'p') {
            v *= 1e3;
          }
          break;
        case 'q':
        case 'Q':
          v = p2 ().y();
          if (code == 'q') {
            v *= 1e3;
          }
          break;
        case 'x':
        case 'X':
          v = dx;
          if (code == 'x') {
            v *= 1e3;
          }
          break;
        case 'y':
        case 'Y':
          v = dy;
          if (code == 'y') {
            v *= 1e3;
          }
          break;
        case 'a':
        case 'A':
          v = fabs (dx) * fabs (dy);
          if (code == 'A') {
            v *= 1e-6;  //  area in square mm
          }
          break;
        }

        const char *ff = 0;
        if (*f == '{') {
          ff = strchr (f + 1, '}');
          if (ff) {
            ++f;
          }
        } 

        if (ff) {
          std::string sfmt (f, 0, ff - f);
          r += tl::sprintf (sfmt.c_str (), v);
          f = ff + 1;
        } else {
          r += tl::sprintf ("%.8g", v);
        }

      }

    }

  }

  return r;
}

} // namespace ant

