
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layPlugin.h"
#include "layMainWindow.h"
#include "layAbstractMenu.h"
#include "antConfigPage.h"
#include "antConfig.h"

namespace ant
{

class PluginDeclaration
  : public lay::PluginDeclaration
{
public:
  PluginDeclaration () 
    : m_current_template (0)
  {
    //  .. nothing yet ..
  }
  
  virtual void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    options.push_back (std::pair<std::string, std::string> (cfg_max_number_of_rulers, "-1"));
    options.push_back (std::pair<std::string, std::string> (cfg_ruler_snap_range, "8"));
    options.push_back (std::pair<std::string, std::string> (cfg_ruler_color, lay::ColorConverter ().to_string (QColor ())));
    options.push_back (std::pair<std::string, std::string> (cfg_ruler_halo, "true"));
    options.push_back (std::pair<std::string, std::string> (cfg_ruler_snap_mode, ACConverter ().to_string (ant::Object::Any)));
    options.push_back (std::pair<std::string, std::string> (cfg_ruler_obj_snap, tl::to_string (true)));
    options.push_back (std::pair<std::string, std::string> (cfg_ruler_grid_snap, tl::to_string (false)));
    options.push_back (std::pair<std::string, std::string> (cfg_ruler_templates, ""));
    options.push_back (std::pair<std::string, std::string> (cfg_current_ruler_template, "0"));
    //  grid-micron is not configured here since some other entity is supposed to do this.
  }

  virtual lay::ConfigPage *config_page (QWidget *parent, std::string &title) const
  {
    title = tl::translate ("Rulers And Markers");
    return new ant::ConfigPage (parent); 
  }

  virtual void get_menu_entries (std::vector<lay::MenuEntry> &menu_entries) const
  {
    lay::PluginDeclaration::get_menu_entries (menu_entries);
    menu_entries.push_back (lay::MenuEntry ("rulers_group", "edit_menu.end"));
    menu_entries.push_back (lay::MenuEntry ("ant::clear_all_rulers", "clear_all_rulers:edit", "edit_menu.end", tl::translate ("Clear All Rulers And Markers(Ctrl+K)")));
    menu_entries.push_back (lay::MenuEntry ("ant::configure", "configure_rulers", "edit_menu.end", tl::translate ("Ruler And Marker Setup")));
  }

  virtual lay::Plugin *create_plugin (db::Manager *manager, lay::MainWindow *, lay::LayoutView *view) const
  {
    return new ant::Service (manager, view);
  }

  virtual bool implements_editable (std::string &title) const
  {
    title = tl::translate ("Rulers And Markers");
    return true;
  }
  
  virtual bool implements_mouse_mode (std::string &title) const
  {
    title = "ruler:ruler_mode_group:ruler_templates_group\t<:ruler.png>";
    return true;
  }
  
  virtual bool configure (const std::string &name, const std::string &value)
  {
    if (name == cfg_ruler_templates) {

      m_templates = ant::Template::from_string (value);
      
    } else if (name == cfg_current_ruler_template) {
      
      int n = 0;
      tl::from_string (value, n);
      m_current_template = n;
      
    }
    return false;
  }

  virtual void config_finalize ()
  {
    if (m_templates.empty ()) {
      m_templates.push_back (Template ());
    }
  
    if (m_current_template < 0 || m_current_template >= int (m_templates.size ())) {
      m_current_template = 0;
    }
      
    std::vector<std::string> menu_entries = lay::MainWindow::instance ().menu ().group ("ruler_mode_group");
    for (std::vector<std::string>::const_iterator m = menu_entries.begin (); m != menu_entries.end (); ++m) {
      lay::Action action = lay::MainWindow::instance ().menu ().action (*m);
      action.set_title (m_templates [m_current_template].title ());
    }
    
    std::vector<std::string> tmpl_group = lay::MainWindow::instance ().menu ().group ("ruler_templates_group");
    for (std::vector<std::string>::const_iterator t = tmpl_group.begin (); t != tmpl_group.end (); ++t) {
      std::vector<std::string> items = lay::MainWindow::instance ().menu ().items (*t);        
      for (std::vector<std::string>::const_iterator i = items.begin (); i != items.end (); ++i) {
        lay::MainWindow::instance ().menu ().delete_item (*i);
      }
    }
      
    m_actions.clear ();
    if (m_templates.size () > 1) {
      int it = 0;
      for (std::vector<Template>::const_iterator tt = m_templates.begin (); tt != m_templates.end (); ++tt, ++it) {
        m_actions.push_back (lay::ConfigureAction (tt->title (), cfg_current_ruler_template, tl::to_string (it)));
        m_actions.back ().set_checkable (true);
        m_actions.back ().set_checked (it == m_current_template);
        for (std::vector<std::string>::const_iterator t = tmpl_group.begin (); t != tmpl_group.end (); ++t) {
          lay::MainWindow::instance ().menu ().insert_item (*t + ".end", "ruler_template_" + tl::to_string (it), m_actions.back ());
        }
      }
    }
      
  }
  
private:
  std::vector<ant::Template> m_templates;
  int m_current_template;
  std::vector<lay::ConfigureAction> m_actions;
};

static tl::Registrar<lay::PluginDeclaration>::Class config_decl (new ant::PluginDeclaration (), 3000, "ant::Plugin");

}

