
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/



#ifndef HDR_dbOASISWriter
#define HDR_dbOASISWriter

#include "dbWriter.h"
#include "dbOASIS.h"

namespace tl
{
  class OutputStream;
}

namespace db
{

class Layout;
class SaveLayoutOptions;
class OASISArray;

/**
 *  @brief A OASIS writer abstraction
 */

class OASISWriter
  : public db::WriterBase
{
public:
  /**
   *  @brief Instantiate the writer
   */
  OASISWriter ();

  /**
   *  @brief Write the layout object
   */
  void write (const db::Layout &layout, tl::OutputStream &stream, const db::SaveLayoutOptions &options) throw (tl::Exception);

private:
  tl::OutputStream *mp_stream;
  double m_sf;

  void write_byte (unsigned char b);

  void write_astring (const char *s);
  void write_bstring (const char *s);
  void write_nstring (const char *s);

  void write_gdelta (const db::Point &p);
  void write_coord (db::Coord c);
  void write_ucoord (db::Coord c);

  void write (double d);
  void write (float d);
  void write (long n);
  void write (unsigned long n);

  void write (int n)
  {
    write (long (n));
  }

  void write (unsigned int n)
  {
    write ((unsigned long) (n));
  }

  void write (const Repetition &rep);

  void emit_property_def (const db::Layout &layout, db::PropertiesRepository::properties_id_type prop_id, unsigned int &propname_id, unsigned int &propstring_id);
  void write_insts (const db::Layout &layout, const db::Cell &cell);
  void write_texts (const db::Layout &layout, const db::Cell &cell, const db::LayerProperties &lprops, unsigned int layer);
  void write_polygons (const db::Layout &layout, const db::Cell &cell, const db::LayerProperties &lprops, unsigned int layer);
  void write_boxes (const db::Layout &layout, const db::Cell &cell, const db::LayerProperties &lprops, unsigned int layer);
  void write_paths (const db::Layout &layout, const db::Cell &cell, const db::LayerProperties &lprops, unsigned int layer);
  void write_edges (const db::Layout &layout, const db::Cell &cell, const db::LayerProperties &lprops, unsigned int layer);

  void write_props (const db::Layout &layout, const db::PropertiesRepository::properties_set &props);
  void write_pointlist (const std::vector<db::Point> &pointlist, bool for_polygons);

  std::map <std::string, unsigned long> m_textstrings;
  std::map <std::string, unsigned long> m_propnames;
  std::map <std::string, unsigned long> m_propstrings;

  typedef std::vector<tl::Variant> property_value_list;

  modal_variable<Repetition> mm_repetition;
  modal_variable<db::Layout::cell_index_type> mm_placement_cell;
  modal_variable<db::Coord> mm_placement_x;
  modal_variable<db::Coord> mm_placement_y;
  modal_variable<unsigned int> mm_layer;
  modal_variable<unsigned int> mm_datatype;
  modal_variable<unsigned int> mm_textlayer;
  modal_variable<unsigned int> mm_texttype;
  modal_variable<db::Coord> mm_text_x;
  modal_variable<db::Coord> mm_text_y;
  modal_variable<std::string> mm_text_string;
  modal_variable<db::Coord> mm_geometry_x;
  modal_variable<db::Coord> mm_geometry_y;
  modal_variable<db::Coord> mm_geometry_w;
  modal_variable<db::Coord> mm_geometry_h;
  modal_variable< std::vector<db::Point> > mm_polygon_point_list;
  modal_variable<db::Coord> mm_path_halfwidth;
  modal_variable<db::Coord> mm_path_start_extension;
  modal_variable<db::Coord> mm_path_end_extension;
  modal_variable< std::vector<db::Point> > mm_path_point_list;
  modal_variable<unsigned int> mm_ctrapezoid_type;
  modal_variable<db::Coord> mm_circle_radius;
  modal_variable<std::string> mm_last_property_name;
  modal_variable<bool> mm_last_property_is_sprop;
  modal_variable<property_value_list> mm_last_value_list;
};

} // namespace db

#endif

