
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


//  This header by intention does not have a include guard
//  It is used multiple times inside gsiCallback.h
//
//  It needs these macros to be defined (example for the one argument case)
//   _COUNT        "1"
//   _NAME(x)      "x##1"
//   _TMPLARG      "class A1"
//   _TMPLARGPART  ", class A1"
//   _FUNCARGLIST  "A1"
//   _FUNCARGLISTPART  ", A1"
//   _ADDARGS      "this->template add_arg<A1> ();"
//   _GETARGVARS   "A1 a1 = args.template get_value<A1> (typename type_traits<A1>::tag ());"
//   _ARGVARLIST   "a1"
//   _ARGVARLISTPART   ", a1"

template <class X _TMPLARGPART>
class _NAME(MethodVoid)
  : public MethodSpecificBase <X>
{
public:
  _NAME(MethodVoid) (const std::string &name, void (X::*m) (_FUNCARGLIST), const std::string &doc, gsi::Callback X::*cb = 0)
    : MethodSpecificBase <X> (name, doc, false, false, cb), m_m (m)
  { 
    _ADDARGS
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(MethodVoid) (this->name (), m_m, this->doc (), this->callback ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS;
    (((X *)cls)->*m_m) (_ARGVARLIST);
  }

private:
   void (X::*m_m) (_FUNCARGLIST);
};

template <class X _TMPLARGPART>
class _NAME(ConstMethodVoid)
  : public MethodSpecificBase <X>
{
public:
  _NAME(ConstMethodVoid) (const std::string &name, void (X::*m) (_FUNCARGLIST) const, const std::string &doc, gsi::Callback X::*cb = 0)
    : MethodSpecificBase <X> (name, doc, true, false, cb), m_m (m)
  { 
    _ADDARGS
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(ConstMethodVoid) (this->name (), m_m, this->doc (), this->callback ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS;
    (((const X *)cls)->*m_m) (_ARGVARLIST);
  }

private:
   void (X::*m_m) (_FUNCARGLIST) const;
};

template <class X _TMPLARGPART>
class _NAME(ExtMethodVoid)
  : public MethodSpecificBase <X>
{
public:
  _NAME(ExtMethodVoid) (const std::string &name, void (*xm) (X * _FUNCARGLISTPART), const std::string &doc, gsi::Callback X::*cb = 0)
    : MethodSpecificBase <X> (name, doc, is_const_x<X>::value (), false, cb), m_xm (xm)
  { 
    _ADDARGS
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(ExtMethodVoid) (this->name (), m_xm, this->doc (), this->callback ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS;
    (*m_xm) ((X *)cls _ARGVARLISTPART);
  }

private:
   void (*m_xm) (X * _FUNCARGLISTPART);
};

#if _COUNT != 0
template <_TMPLARG>
#endif
class _NAME(StaticMethodVoid)
  : public StaticMethodBase
{
public:
  _NAME(StaticMethodVoid) (const std::string &name, void (*m) (_FUNCARGLIST), const std::string &doc)
    : StaticMethodBase (name, doc), m_m (m)
  { 
    _ADDARGS
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(StaticMethodVoid) (this->name (), m_m, this->doc ());
  }

  virtual void call (void *, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS;
    (*m_m) (_ARGVARLIST);
  }

private:
   void (*m_m) (_FUNCARGLIST);
};

template <class X, class R _TMPLARGPART>
class _NAME(Method)
  : public MethodSpecificBase <X>
{
public:
  _NAME(Method) (const std::string &name, R (X::*m) (_FUNCARGLIST), const std::string &doc, gsi::Callback X::*cb = 0)
    : MethodSpecificBase <X> (name, doc, false, false, cb), m_m (m)
  { 
    _ADDARGS
    this->template set_return<R> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(Method) (this->name (), m_m, this->doc (), this->callback ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS;
    ret.set_value (typename type_traits<R>::tag (), (((X *)cls)->*m_m) (_ARGVARLIST));
  }

private:
   R (X::*m_m) (_FUNCARGLIST);
};

template <class X, class R _TMPLARGPART>
class _NAME(ConstMethod)
  : public MethodSpecificBase <X>
{
public:
  _NAME(ConstMethod) (const std::string &name, R (X::*m) (_FUNCARGLIST) const, const std::string &doc, gsi::Callback X::*cb = 0)
    : MethodSpecificBase <X> (name, doc, true, false, cb), m_m (m)
  { 
    _ADDARGS
    this->template set_return<R> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(ConstMethod) (this->name (), m_m, this->doc (), this->callback ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS;
    ret.set_value (typename type_traits<R>::tag (), (((const X *)cls)->*m_m) (_ARGVARLIST));
  }

private:
   R (X::*m_m) (_FUNCARGLIST) const;
};

template <class X, class R _TMPLARGPART>
class _NAME(ExtMethod)
  : public MethodSpecificBase <X>
{
public:
  _NAME(ExtMethod) (const std::string &name, R (*xm) (X * _FUNCARGLISTPART), const std::string &doc, gsi::Callback X::*cb = 0)
    : MethodSpecificBase <X> (name, doc, is_const_x<X>::value (), false, cb), m_xm (xm)
  { 
    _ADDARGS
    this->template set_return<R> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(ExtMethod) (this->name (), m_xm, this->doc (), this->callback ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS;
    ret.set_value (typename type_traits<R>::tag (), (*m_xm) ((X *)cls _ARGVARLISTPART));
  }

private:
   R (*m_xm) (X * _FUNCARGLISTPART);
};

template <class R _TMPLARGPART>
class _NAME(StaticMethod)
  : public StaticMethodBase
{
public:
  _NAME(StaticMethod) (const std::string &name, R (*m) (_FUNCARGLIST), const std::string &doc)
    : StaticMethodBase (name, doc), m_m (m)
  { 
    _ADDARGS
    this->template set_return<R> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(StaticMethod) (this->name (), m_m, this->doc ());
  }

  virtual void call (void *, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS;
    ret.set_value (typename type_traits<R>::tag (), (*m_m) (_ARGVARLIST));
  }

private:
   R (*m_m) (_FUNCARGLIST);
};

template <class X _TMPLARGPART>
class _NAME(Constructor)
  : public StaticMethodBase
{
public:
  _NAME(Constructor) (const std::string &name, X *(*m) (_FUNCARGLIST), const std::string &doc)
    : StaticMethodBase (name, doc), m_m (m)
  { 
    _ADDARGS
    this->template set_return_new<X> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(Constructor) (this->name (), m_m, this->doc ());
  }

  virtual void call (void *, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS;
    ret.set_value (typename type_traits<X *>::tag (), (*m_m) (_ARGVARLIST));
  }

private:
   X *(*m_m) (_FUNCARGLIST);
};

//  pointer iterator method descriptors

template <class X, class R _TMPLARGPART>
class _NAME(MethodPtrIter) 
  : public MethodSpecificBase <X>
{
public:
  typedef R &return_type;
  typedef R value_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef IterPtrAdaptor<return_type, value_type> iter_adaptor_type;

  _NAME(MethodPtrIter) (const std::string &name, R *(X::*b) (_FUNCARGLIST), R *(X::*e) (_FUNCARGLIST), const std::string &doc)
    : MethodSpecificBase <X> (name, doc, false, false, 0), m_b (b), m_e (e)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(MethodPtrIter) (this->name (), m_b, m_e, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((((X *)cls)->*m_b) (_ARGVARLIST), (((X *)cls)->*m_e) (_ARGVARLIST))));
  }

private:
   R *(X::*m_b) (_FUNCARGLIST);
   R *(X::*m_e) (_FUNCARGLIST);
};

template <class X, class R _TMPLARGPART>
class _NAME(ConstMethodPtrIter) 
  : public MethodSpecificBase <X>
{
public:
  typedef R &return_type;
  typedef R value_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef IterPtrAdaptor<return_type, value_type> iter_adaptor_type;

  _NAME(ConstMethodPtrIter) (const std::string &name, R *(X::*b) (_FUNCARGLIST) const, R *(X::*e) (_FUNCARGLIST) const, const std::string &doc)
    : MethodSpecificBase <X> (name, doc, true, false, 0), m_b (b), m_e (e)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(ConstMethodPtrIter) (this->name (), m_b, m_e, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((((const X *)cls)->*m_b) (_ARGVARLIST), (((const X *)cls)->*m_e) (_ARGVARLIST))));
  }

private:
   R *(X::*m_b) (_FUNCARGLIST) const;
   R *(X::*m_e) (_FUNCARGLIST) const;
};

template <class X, class R _TMPLARGPART>
class _NAME(ExtMethodPtrIter) 
  : public MethodSpecificBase <X>
{
public:
  typedef R &return_type;
  typedef R value_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef IterPtrAdaptor<return_type, value_type> iter_adaptor_type;

  _NAME(ExtMethodPtrIter) (const std::string &name, R *(*xb) (X * _FUNCARGLISTPART), R *(*xe) (X * _FUNCARGLISTPART), const std::string &doc)
    : MethodSpecificBase <X> (name, doc, is_const_x<X>::value (), false, 0), m_xb (xb), m_xe (xe)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(ExtMethodPtrIter) (this->name (), m_xb, m_xe, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((*m_xb) ((X *)cls _ARGVARLISTPART), (*m_xe) ((X *)cls _ARGVARLISTPART))));
  }

private:
   R *(*m_xb) (X * _FUNCARGLISTPART);
   R *(*m_xe) (X * _FUNCARGLISTPART);
};

template <class R _TMPLARGPART>
class _NAME(StaticMethodPtrIter) 
  : public StaticMethodBase
{
public:
  typedef R &return_type;
  typedef R value_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef IterPtrAdaptor<return_type, value_type> iter_adaptor_type;

  _NAME(StaticMethodPtrIter) (const std::string &name, R *(*b) (_FUNCARGLIST), R *(*e) (_FUNCARGLIST), const std::string &doc)
    : StaticMethodBase (name, doc), m_b (b), m_e (e)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(StaticMethodPtrIter) (this->name (), m_b, m_e, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((*m_b) (_ARGVARLIST), ((*m_e) (_ARGVARLIST)))));
  }

private:
   R *(*m_b) (_FUNCARGLIST);
   R *(*m_e) (_FUNCARGLIST);
};

//  pair iterator method descriptors

template <class X, class I _TMPLARGPART>
class _NAME(MethodBiIter) 
  : public MethodSpecificBase <X>
{
public:
  typedef typename std::iterator_traits<I> it_traits;
  typedef typename it_traits::value_type value_type;
  typedef typename it_traits::reference return_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef IterAdaptor<I, return_type, value_type> iter_adaptor_type;

  _NAME(MethodBiIter) (const std::string &name, I (X::*b) (_FUNCARGLIST), I (X::*e) (_FUNCARGLIST), const std::string &doc)
    : MethodSpecificBase <X> (name, doc, false, false, 0), m_b (b), m_e (e)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(MethodBiIter) (this->name (), m_b, m_e, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((((X *)cls)->*m_b) (_ARGVARLIST), (((X *)cls)->*m_e) (_ARGVARLIST))));
  }

private:
   I (X::*m_b) (_FUNCARGLIST);
   I (X::*m_e) (_FUNCARGLIST);
};

template <class X, class I _TMPLARGPART>
class _NAME(ConstMethodBiIter) 
  : public MethodSpecificBase <X>
{
public:
  typedef typename std::iterator_traits<I> it_traits;
  typedef typename it_traits::reference return_type;
  typedef typename it_traits::value_type value_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef IterAdaptor<I, return_type, value_type> iter_adaptor_type;

  _NAME(ConstMethodBiIter) (const std::string &name, I (X::*b) (_FUNCARGLIST) const, I (X::*e) (_FUNCARGLIST) const, const std::string &doc)
    : MethodSpecificBase <X> (name, doc, true, false, 0), m_b (b), m_e (e)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(ConstMethodBiIter) (this->name (), m_b, m_e, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((((const X *)cls)->*m_b) (_ARGVARLIST), (((const X *)cls)->*m_e) (_ARGVARLIST))));
  }

private:
   I (X::*m_b) (_FUNCARGLIST) const;
   I (X::*m_e) (_FUNCARGLIST) const;
};

template <class X, class I _TMPLARGPART>
class _NAME(ExtMethodBiIter) 
  : public MethodSpecificBase <X>
{
public:
  typedef typename std::iterator_traits<I> it_traits;
  typedef typename it_traits::reference return_type;
  typedef typename it_traits::value_type value_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef IterAdaptor<I, return_type, value_type> iter_adaptor_type;

  _NAME(ExtMethodBiIter) (const std::string &name, I (*xb) (X * _FUNCARGLISTPART), I (*xe) (X * _FUNCARGLISTPART), const std::string &doc)
    : MethodSpecificBase <X> (name, doc, is_const_x<X>::value (), false, 0), m_xb (xb), m_xe (xe)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(ExtMethodBiIter) (this->name (), m_xb, m_xe, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((*m_xb) ((X *)cls _ARGVARLISTPART), (*m_xe) ((X *)cls _ARGVARLISTPART))));
  }

private:
   I (*m_xb) (X * _FUNCARGLISTPART);
   I (*m_xe) (X * _FUNCARGLISTPART);
};

template <class I _TMPLARGPART>
class _NAME(StaticMethodBiIter) 
  : public StaticMethodBase
{
public:
  typedef typename std::iterator_traits<I> it_traits;
  typedef typename it_traits::reference return_type;
  typedef typename it_traits::value_type value_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef IterAdaptor<I, return_type, value_type> iter_adaptor_type;

  _NAME(StaticMethodBiIter) (const std::string &name, I (*b) (_FUNCARGLIST), I (*e) (_FUNCARGLIST), const std::string &doc)
    : StaticMethodBase (name, doc), m_b (b), m_e (e)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(StaticMethodBiIter) (this->name (), m_b, m_e, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((*m_b) (_ARGVARLIST), ((*m_e) (_ARGVARLIST)))));
  }

private:
   I (*m_b) (_FUNCARGLIST);
   I (*m_e) (_FUNCARGLIST);
};

//  free iterator method descriptors

template <class X, class I _TMPLARGPART>
class _NAME(MethodFreeIter) 
  : public MethodSpecificBase <X>
{
public:
  typedef typename std::iterator_traits<I> it_traits;
  typedef typename it_traits::value_type value_type;
  typedef typename it_traits::reference return_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef FreeIterAdaptor<I, return_type, value_type> iter_adaptor_type;

  _NAME(MethodFreeIter) (const std::string &name, I (X::*i) (_FUNCARGLIST), const std::string &doc)
    : MethodSpecificBase <X> (name, doc, false, false, 0), m_i (i)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(MethodFreeIter) (this->name (), m_i, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((((X *)cls)->*m_i) (_ARGVARLIST))));
  }

private:
   I (X::*m_i) (_FUNCARGLIST);
};

template <class X, class I _TMPLARGPART>
class _NAME(ConstMethodFreeIter) 
  : public MethodSpecificBase <X>
{
public:
  typedef typename std::iterator_traits<I> it_traits;
  typedef typename it_traits::value_type value_type;
  typedef typename it_traits::reference return_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef FreeIterAdaptor<I, return_type, value_type> iter_adaptor_type;

  _NAME(ConstMethodFreeIter) (const std::string &name, I (X::*i) (_FUNCARGLIST) const, const std::string &doc)
    : MethodSpecificBase <X> (name, doc, false, false, 0), m_i (i)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(ConstMethodFreeIter) (this->name (), m_i, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((((const X *)cls)->*m_i) (_ARGVARLIST))));
  }

private:
   I (X::*m_i) (_FUNCARGLIST) const;
};

template <class X, class I _TMPLARGPART>
class _NAME(ExtMethodFreeIter) 
  : public MethodSpecificBase <X>
{
public:
  typedef typename std::iterator_traits<I> it_traits;
  typedef typename it_traits::value_type value_type;
  typedef typename it_traits::reference return_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef FreeIterAdaptor<I, return_type, value_type> iter_adaptor_type;

  _NAME(ExtMethodFreeIter) (const std::string &name, I (*xi) (X * _FUNCARGLISTPART), const std::string &doc)
    : MethodSpecificBase <X> (name, doc, is_const_x<X>::value (), false, 0), m_xi (xi)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(ExtMethodFreeIter) (this->name (), m_xi, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((*m_xi) ((X *)cls _ARGVARLISTPART))));
  }

private:
   I (*m_xi) (X * _FUNCARGLISTPART);
};

template <class I _TMPLARGPART>
class _NAME(StaticMethodFreeIter) 
  : public StaticMethodBase
{
public:
  typedef typename std::iterator_traits<I> it_traits;
  typedef typename it_traits::value_type value_type;
  typedef typename it_traits::reference return_type;
  typedef IterAdaptorAbstractBase iter_adaptor_base_type;
  typedef FreeIterAdaptor<I, return_type, value_type> iter_adaptor_type;

  _NAME(StaticMethodFreeIter) (const std::string &name, I (*i) (_FUNCARGLIST), const std::string &doc)
    : StaticMethodBase (name, doc), m_i (i)
  { 
    _ADDARGS
    this->template set_return<iter_adaptor_type> ();
  }

  virtual MethodBase *clone () const 
  {
    return new _NAME(StaticMethodFreeIter) (this->name (), m_i, this->doc ());
  }

  virtual void call (void *cls, SerialArgs &args, SerialArgs &ret) const 
  {
    _GETARGVARS
    ret.set_value (typename type_traits<iter_adaptor_base_type *>::tag (), 
                   static_cast<iter_adaptor_base_type *> (new iter_adaptor_type ((*m_i) (_ARGVARLIST))));
  }

private:
   I (*m_i) (_FUNCARGLIST);
};

template <class X _TMPLARGPART>
Methods
method (const std::string &name, void (X::*m) (_FUNCARGLIST), const std::string &doc = std::string ())
{
  return Methods (new _NAME(MethodVoid) <X _FUNCARGLISTPART> (name, m, doc));
}

template <class X _TMPLARGPART>
Methods
method_ext (const std::string &name, void (*xm) (X * _FUNCARGLISTPART), const std::string &doc = std::string ())
{
  return Methods (new _NAME(ExtMethodVoid) <X _FUNCARGLISTPART> (name, xm, doc));
}

#if _COUNT != 0
template <_TMPLARG>
#else
inline
#endif
Methods
method (const std::string &name, void (*m) (_FUNCARGLIST), const std::string &doc = std::string ())
{
#if _COUNT != 0
  return Methods (new _NAME(StaticMethodVoid) <_FUNCARGLIST> (name, m, doc));
#else
  return Methods (new _NAME(StaticMethodVoid) (name, m, doc));
#endif
}

template <class X _TMPLARGPART>
Methods
callback (const std::string &name, void (X::*m) (_FUNCARGLIST), Callback X::*cb, const std::string &doc = std::string ())
{
  return Methods (new _NAME(MethodVoid) <X _FUNCARGLISTPART> (name, m, doc, cb));
}

template <class X _TMPLARGPART>
Methods
method (const std::string &name, void (X::*m) (_FUNCARGLIST) const, const std::string &doc = std::string ())
{
  return Methods (new _NAME(ConstMethodVoid) <X _FUNCARGLISTPART> (name, m, doc));
}

template <class X _TMPLARGPART>
Methods
callback (const std::string &name, void (X::*m) (_FUNCARGLIST) const, Callback X::*cb, const std::string &doc = std::string ())
{
  return Methods (new _NAME(ConstMethodVoid) <X _FUNCARGLISTPART> (name, m, doc, cb));
}

template <class X, class R _TMPLARGPART>
Methods
method (const std::string &name, R (X::*m) (_FUNCARGLIST), const std::string &doc = std::string ())
{
  return Methods (new _NAME(Method) <X, R _FUNCARGLISTPART> (name, m, doc));
}

template <class X, class R _TMPLARGPART>
Methods
method_ext (const std::string &name, R (*xm) (X * _FUNCARGLISTPART), const std::string &doc = std::string ())
{
  return Methods (new _NAME(ExtMethod) <X, R _FUNCARGLISTPART> (name, xm, doc));
}

template <class X _TMPLARGPART>
Methods
constructor (const std::string &name, X *(*m) (_FUNCARGLIST), const std::string &doc = std::string ())
{
  return Methods (new _NAME(Constructor) <X _FUNCARGLISTPART> (name, m, doc));
}

template <class R _TMPLARGPART>
Methods
method (const std::string &name, R (*m) (_FUNCARGLIST), const std::string &doc = std::string ())
{
  return Methods (new _NAME(StaticMethod) <R _FUNCARGLISTPART> (name, m, doc));
}

template <class X, class R _TMPLARGPART>
Methods
callback (const std::string &name, R (X::*m) (_FUNCARGLIST), Callback X::*cb, const std::string &doc = std::string ())
{
  return Methods (new _NAME(Method) <X, R _FUNCARGLISTPART> (name, m, doc, cb));
}

template <class X, class R _TMPLARGPART>
Methods
method (const std::string &name, R (X::*m) (_FUNCARGLIST) const, const std::string &doc = std::string ())
{
  return Methods (new _NAME(ConstMethod) <X, R _FUNCARGLISTPART> (name, m, doc));
}

template <class X, class R _TMPLARGPART>
Methods
callback (const std::string &name, R (X::*m) (_FUNCARGLIST) const, Callback X::*cb, const std::string &doc = std::string ())
{
  return Methods (new _NAME(ConstMethod) <X, R _FUNCARGLISTPART> (name, m, doc, cb));
}

//  pointer iterators

template <class R _TMPLARGPART>
Methods
iterator (const std::string &name, R *(*b) (_FUNCARGLIST), R *(*e) (_FUNCARGLIST), const std::string &doc = std::string ())
{
  return Methods (new _NAME(StaticMethodPtrIter) <R _FUNCARGLISTPART> (name, b, e, doc));
}

template <class X, class R _TMPLARGPART>
Methods
iterator (const std::string &name, R *(X::*b) (_FUNCARGLIST), R *(X::*e) (_FUNCARGLIST), const std::string &doc = std::string ())
{
  return Methods (new _NAME(MethodPtrIter) <X, R _FUNCARGLISTPART> (name, b, e, doc));
}

template <class X, class R _TMPLARGPART>
Methods
iterator_ext (const std::string &name, R *(*xb) (X * _FUNCARGLISTPART), R *(*xe) (X * _FUNCARGLISTPART), const std::string &doc = std::string ())
{
  return Methods (new _NAME(ExtMethodPtrIter) <X, R _FUNCARGLISTPART> (name, xb, xe, doc));
}

template <class X, class R _TMPLARGPART>
Methods
iterator (const std::string &name, R *(X::*b) (_FUNCARGLIST) const, R *(X::*e) (_FUNCARGLIST) const, const std::string &doc = std::string ())
{
  return Methods (new _NAME(ConstMethodPtrIter) <X, R _FUNCARGLISTPART> (name, b, e, doc));
}

//  pair iterators

template <class X, class I _TMPLARGPART>
Methods
_iterator (const std::string &name, I (X::*b) (_FUNCARGLIST), I (X::*e) (_FUNCARGLIST), const std::string &doc)
{
  return Methods (new _NAME(MethodBiIter) <X, I _FUNCARGLISTPART> (name, b, e, doc));
}

template <class X, class I  _TMPLARGPART>
Methods
_iterator_ext (const std::string &name, I (*xb) (X * _FUNCARGLISTPART), I (*xe) (X * _FUNCARGLISTPART), const std::string &doc)
{
  return Methods (new _NAME(ExtMethodBiIter) <X, I _FUNCARGLISTPART> (name, xb, xe, doc));
}

template <class I _TMPLARGPART>
Methods
_iterator (const std::string &name, I (*b) (_FUNCARGLIST), I (*e) (_FUNCARGLIST), const std::string &doc)
{
  return Methods (new _NAME(StaticMethodBiIter) <I _FUNCARGLISTPART> (name, b, e, doc));
}

template <class I _TMPLARGPART>
Methods
iterator (const std::string &name, I (*b) (_FUNCARGLIST), I (*e) (_FUNCARGLIST), const std::string &doc = std::string ())
{
  return _iterator (name, b, e, doc);
}

template <class X, class I _TMPLARGPART>
Methods
iterator (const std::string &name, I (X::*b) (_FUNCARGLIST), I (X::*e) (_FUNCARGLIST), const std::string &doc = std::string ())
{
  return _iterator (name, b, e, doc);
}

template <class X, class I _TMPLARGPART>
Methods
iterator_ext (const std::string &name, I (*xb) (X * _FUNCARGLISTPART), I (*xe) (X * _FUNCARGLISTPART), const std::string &doc = std::string ())
{
  return _iterator_ext (name, xb, xe, doc);
}

template <class X, class I _TMPLARGPART>
Methods
_iterator (const std::string &name, I (X::*b) (_FUNCARGLIST) const, I (X::*e) (_FUNCARGLIST) const, const std::string &doc)
{
  return Methods (new _NAME(ConstMethodBiIter) <X, I _FUNCARGLISTPART> (name, b, e, doc));
}

template <class X, class I _TMPLARGPART>
Methods
iterator (const std::string &name, I (X::*b) (_FUNCARGLIST) const, I (X::*e) (_FUNCARGLIST) const, const std::string &doc = std::string ())
{
  return _iterator (name, b, e, doc);
}

//  free iterators

template <class X, class I  _TMPLARGPART>
Methods
_iterator (const std::string &name, I (X::*i) (_FUNCARGLIST), const std::string &doc)
{
  return Methods (new _NAME(MethodFreeIter) <X, I _FUNCARGLISTPART> (name, i, doc));
}

template <class X, class I  _TMPLARGPART>
Methods
_iterator_ext (const std::string &name, I (*xi) (X * _FUNCARGLISTPART), const std::string &doc)
{
  return Methods (new _NAME(ExtMethodFreeIter) <X, I _FUNCARGLISTPART> (name, xi, doc));
}

template <class I _TMPLARGPART>
Methods
_iterator (const std::string &name, I (*i) (_FUNCARGLIST), const std::string &doc)
{
  return Methods (new _NAME(StaticMethodFreeIter) <I _FUNCARGLISTPART> (name, i, doc));
}

template <class I _TMPLARGPART>
Methods
iterator (const std::string &name, I (*i) (_FUNCARGLIST), const std::string &doc = std::string ())
{
  return _iterator (name, i, doc);
}

template <class X, class I _TMPLARGPART>
Methods
iterator (const std::string &name, I (X::*i) (_FUNCARGLIST), const std::string &doc = std::string ())
{
  return _iterator (name, i, doc);
}

template <class X, class I _TMPLARGPART>
Methods
iterator_ext (const std::string &name, I (*xi) (X * _FUNCARGLISTPART), const std::string &doc = std::string ())
{
  return _iterator_ext (name, xi, doc);
}

template <class X, class I _TMPLARGPART>
Methods
_iterator (const std::string &name, I (X::*i) (_FUNCARGLIST) const, const std::string &doc)
{
  return Methods (new _NAME(ConstMethodFreeIter) <X, I _FUNCARGLISTPART> (name, i, doc));
}

template <class X, class I _TMPLARGPART>
Methods
iterator (const std::string &name, I (X::*i) (_FUNCARGLIST) const, const std::string &doc = std::string ())
{
  return _iterator (name, i, doc);
}


