
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <QtGui/QFileDialog>
#include <QtCore/QFileInfo>

#include "layFileDialog.h"
#include "tlInternational.h"

namespace lay
{

FileDialog::FileDialog (QWidget *parent, const std::string &title, const std::string &filters, const std::string &def_suffix)
  : QObject (parent)
{
  m_dir = QDir::current ();
  m_title = title.c_str ();
  m_filters = filters.c_str ();
  m_def_suffix = def_suffix.c_str ();
}

FileDialog::~FileDialog ()
{
  //  .. nothing yet ..
}

static void
split_filters (const QString &filters, QStringList &slist)
{
  QString s;
  for (const char *cp = filters.ascii (); *cp; ++cp) {
    if (cp[0] == ';' && cp[1] == ';') {
      slist << s;
      ++cp;
      s = "";
    } else {
      s += *cp;
    }
  }

  if (s != "") {
    slist << s;
  }
}

bool 
FileDialog::get_open (std::string &fp) 
{
  QString file_name;
  if (! fp.empty ()) {
    QFileInfo fi (fp.c_str ());
    m_dir = fi.absoluteDir ();
    file_name = fi.fileName ();
  }

  QFileDialog fdia;

  fdia.setDirectory (m_dir);
  if (m_def_suffix != "") {
    fdia.setDefaultSuffix (m_def_suffix);
  }

  QStringList types;
  split_filters (m_filters, types);
  fdia.setFilters (types);

  fdia.setReadOnly (true);
  fdia.setViewMode (QFileDialog::Detail);
  fdia.setFileMode (QFileDialog::AnyFile);
  fdia.setAcceptMode (QFileDialog::AcceptOpen);
  fdia.setConfirmOverwrite (true);
  fdia.setCaption (QString (tl::translate ("Load ").c_str ()) + m_title);
  if (! file_name.isEmpty ()) {
    fdia.selectFile (file_name);
  }

  QStringList files;
  if (fdia.exec ()) {
    files = fdia.selectedFiles();
    if (files.size () > 0) {
      fp = files [0].ascii ();
      QFileInfo fi (files [0]);
      m_dir = fi.absoluteDir ();
      return true;
    }
  }

  return false;
}

bool 
FileDialog::get_save (std::string &fp) 
{
  QString file_name;
  if (! fp.empty ()) {
    QFileInfo fi (fp.c_str ());
    m_dir = fi.absoluteDir ();
    file_name = fi.fileName ();
  }

  QFileDialog fdia;

  fdia.setDirectory (m_dir);
  if (m_def_suffix != "") {
    fdia.setDefaultSuffix (m_def_suffix);
  }

  QStringList types;
  split_filters (m_filters, types);
  fdia.setFilters (types);

  fdia.setReadOnly (false);
  fdia.setViewMode (QFileDialog::Detail);
  fdia.setFileMode (QFileDialog::AnyFile);
  fdia.setAcceptMode (QFileDialog::AcceptSave);
  fdia.setConfirmOverwrite (true);
  fdia.setCaption (QString (tl::translate ("Save ").c_str ()) + m_title);
  if (! file_name.isEmpty ()) {
    fdia.selectFile (file_name);
  }

  QStringList files;
  if (fdia.exec ()) {
    files = fdia.selectedFiles();
    if (files.size () > 0) {
      fp = files [0].ascii ();
      QFileInfo fi (files [0]);
      m_dir = fi.absoluteDir ();
      return true;
    }
  }

  return false;
}

} // namespace lay

