
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#ifndef HDR_layLayoutViewConfigPages
#define HDR_layLayoutViewConfigPages

#include "layPlugin.h"
#include "layColorPalette.h"
#include "dbObject.h"

namespace Ui {
  class LayoutViewConfigPage;
  class LayoutViewConfigPage2;
  class LayoutViewConfigPage3;
  class LayoutViewConfigPage4;
  class LayoutViewConfigPage5;
}

namespace lay
{

class ColorButton;

// -------------------------------------------------------------
//  This file provides the main declarations for the configuration
//  dialogs for the layout view.

class LayoutViewConfigPage 
  : public lay::ConfigPage
{
Q_OBJECT

public:
  LayoutViewConfigPage (QWidget *parent);
  ~LayoutViewConfigPage ();

  virtual void setup (lay::Plugin *root);
  virtual void commit (lay::Plugin *root);

private:
  Ui::LayoutViewConfigPage *mp_ui;
};

class LayoutViewConfigPage2 
  : public lay::ConfigPage
{
Q_OBJECT

public:
  LayoutViewConfigPage2 (QWidget *parent);
  ~LayoutViewConfigPage2 ();

  virtual void setup (lay::Plugin *root);
  virtual void commit (lay::Plugin *root);

private:
  Ui::LayoutViewConfigPage2 *mp_ui;
  lay::ColorButton *mp_text_color_cbtn;
  lay::ColorButton *mp_box_color_cbtn;
};

class LayoutViewConfigPage3 
  : public lay::ConfigPage
{
Q_OBJECT

public:
  LayoutViewConfigPage3 (QWidget *parent);
  ~LayoutViewConfigPage3 ();

  virtual void setup (lay::Plugin *root);
  virtual void commit (lay::Plugin *root);

private:
  Ui::LayoutViewConfigPage3 *mp_ui;
};

class LayoutViewConfigPage4 
  : public lay::ConfigPage, 
    public db::Object
{
Q_OBJECT

public:
  LayoutViewConfigPage4 (QWidget *parent);
  ~LayoutViewConfigPage4 ();

  virtual void setup (lay::Plugin *root);
  virtual void commit (lay::Plugin *root);

  virtual void undo (db::Op *op) throw (); 
  virtual void redo (db::Op *op) throw ();

public slots:
  void color_button_clicked ();
  void undo_button_clicked ();
  void redo_button_clicked ();
  void reset_button_clicked ();
  void edit_order_changed (int s);
  
private:
  Ui::LayoutViewConfigPage4 *mp_ui;
  lay::ColorPalette m_palette;
  db::Manager m_manager;
  bool m_edit_order_changed_disabled;

  void update ();
  void set_edit_order (bool edit_order);
};

class LayoutViewConfigPage5 
  : public lay::ConfigPage
{
Q_OBJECT

public:
  LayoutViewConfigPage5 (QWidget *parent);
  ~LayoutViewConfigPage5 ();

  virtual void setup (lay::Plugin *root);
  virtual void commit (lay::Plugin *root);

public slots:
  void browse_clicked ();

private:
  Ui::LayoutViewConfigPage5 *mp_ui;
};

}

#endif

