
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/



#include "layMove.h"
#include "layEditable.h"
#include "layLayoutView.h"

namespace lay
{

// -------------------------------------------------------------
//  MoveService implementation

MoveService::MoveService (lay::LayoutView *view)
  : lay::ViewService (view->view_object_widget ()), 
    m_dragging (false),
    mp_editables (view)
{ 
  // .. nothing yet ..
}

MoveService::~MoveService ()
{
  drag_cancel ();
}

bool 
MoveService::mouse_move_event (const db::DPoint &p, bool prio) 
{
  if (m_dragging) {
    mp_editables->move (p);
  }
  return false;  // not taken to allow the mouse tracker to receive events as well
}

bool 
MoveService::mouse_click_event (const db::DPoint &p, unsigned int buttons, bool prio)
{
  if (mouse_press_event (p, buttons, prio)) {
    return true;
  } else if (prio && (buttons & lay::RightButton) != 0 && m_dragging) {
    mp_editables->move_transform (p, db::fixpoint_trans::r90);
    return true;
  } else {
    return false;
  }
}

bool 
MoveService::mouse_press_event (const db::DPoint &p, unsigned int buttons, bool prio)
{
  if (prio && (buttons & lay::LeftButton) != 0) {

    if (! m_dragging) {
      if (mp_editables->begin_move (p)) {
        m_dragging = true;
        widget ()->grab_mouse (this, false);
      }
    } else {
      m_dragging = false;
      widget ()->ungrab_mouse (this);
      mp_editables->end_move (p);
    }

    return true;

  } else {
    return false;
  }
}

void
MoveService::drag_cancel () 
{ 
  if (m_dragging) {
    mp_editables->edit_cancel ();
    widget ()->ungrab_mouse (this);
    m_dragging = false;
  }
}

}

