
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layRedrawThreadCanvas.h"
#include "layBitmap.h"
#include "layBitmapsToImage.h"
#include "layDrawing.h"

#include <QtGui/QImage>

namespace lay
{

RedrawThreadCanvas::RedrawThreadCanvas ()
{
  // .. nothing yet ..
}

RedrawThreadCanvas::~RedrawThreadCanvas ()
{
  lock ();
  clear_bitmaps ();
  unlock ();
}

void 
RedrawThreadCanvas::set_bitmap (unsigned int n, const lay::Bitmap &bitmap)
{ 
  lock ();
  if (n < mp_bitmap_buffers.size ()) {
    *(mp_bitmap_buffers [n]) = bitmap; 
  }
  unlock ();
}

void 
RedrawThreadCanvas::clear (unsigned int nlayers, unsigned int width, unsigned int height, const lay::Drawings *drawings)
{
  lock ();

  //  delete the buffers and create new ones 
  clear_bitmaps ();

  for (unsigned int i = 0; i < nlayers; ++i) {
    mp_bitmap_buffers.push_back (new lay::Bitmap (width, height));
  }

  for (lay::Drawings::const_iterator d = drawings->begin (); d != drawings->end (); ++d) {
    mp_drawing_bitmap_buffers.push_back (std::vector <lay::Bitmap *> ());
    for (unsigned int i = 0; i < d->num_bitmaps (); ++i) {
      mp_drawing_bitmap_buffers.back ().push_back (new lay::Bitmap (width, height));
    }
  }

  unlock ();
}

void 
RedrawThreadCanvas::set_drawing_bitmap (unsigned int d, unsigned int n, const lay::Bitmap &bitmap)
{ 
  lock ();
  if (d < mp_drawing_bitmap_buffers.size () && n < mp_drawing_bitmap_buffers [d].size ()) {
    *(mp_drawing_bitmap_buffers [d][n]) = bitmap; 
  }
  unlock ();
}

void 
RedrawThreadCanvas::clear_bitmaps ()
{
  while (! mp_bitmap_buffers.empty ()) {
    delete mp_bitmap_buffers.back ();
    mp_bitmap_buffers.pop_back ();
  }

  while (! mp_drawing_bitmap_buffers.empty ()) {
    while (! mp_drawing_bitmap_buffers.back ().empty ()) {
      delete mp_drawing_bitmap_buffers.back ().back ();
      mp_drawing_bitmap_buffers.back ().pop_back ();
    }
    mp_drawing_bitmap_buffers.pop_back ();
  }
}

void 
RedrawThreadCanvas::lock ()
{
  m_mutex.lock ();
}

void 
RedrawThreadCanvas::unlock ()
{
  m_mutex.unlock ();
}

void 
RedrawThreadCanvas::to_image (const std::vector <lay::ViewOp> &view_ops, const lay::DitherPattern &dp, const lay::Drawings *drawings, QImage &img, unsigned int width, unsigned int height)
{
  //  convert the bitmap data to image data
  bitmaps_to_image (view_ops, mp_bitmap_buffers, dp, &img, width, height, true, &m_mutex);

  //  convert the bitmaps of the "drawing" objects too:
  std::vector <std::vector <lay::Bitmap *> >::const_iterator bt = mp_drawing_bitmap_buffers.begin ();
  for (lay::Drawings::const_iterator d = drawings->begin (); d != drawings->end () && bt != mp_drawing_bitmap_buffers.end (); ++d, ++bt) {
    bitmaps_to_image (d->get_view_ops (), *bt, dp, &img, width, height, true, &m_mutex);
  }
}

}

