
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#ifndef HDR_layRedrawThreadCanvas
#define HDR_layRedrawThreadCanvas

#include "dbBox.h"
#include "dbTrans.h"
#include "layViewOp.h"

#include <vector>

#include <QtCore/QMutex>

class QImage;

namespace lay {

class Bitmap;
class Drawings;
class DitherPattern;

class RedrawThreadCanvas
{
public:
  RedrawThreadCanvas ();

  virtual ~RedrawThreadCanvas ();

  /**
   *  @brief Signal that a transfer has been done
   *
   *  This method is called (from the redraw thread) if a transfer has been performed ansynchreously
   */
  virtual void signal_transfer_done () = 0;

  /**
   *  @brief Signal that the drawing has ended
   *
   *  This method is called (from the redraw thread) once the drawing has ended.
   */
  virtual void signal_end_of_drawing () = 0;

  /**
   *  @brief Set a bitmap
   *
   *  This method is called from the redraw thread to transfer data for a certain bitmap.
   *  The bitmaps have been reserved before with clear ().
   */
  virtual void set_bitmap (unsigned int n, const lay::Bitmap &bitmap);

  /**
   *  @brief Clear and prepare the given number of bitmaps
   *
   *  This method is called from RedrawThread::start (), not from the
   *  redraw thread.
   */
  virtual void clear (unsigned int nlayers, unsigned int width, unsigned int height, const lay::Drawings *drawings);

  /**
   *  @brief Set a bitmap for the drawing number d and index n within the drawing.
   *
   *  This method is called from the redraw thread.
   */
  virtual void set_drawing_bitmap (unsigned int d, unsigned int n, const lay::Bitmap &bitmap);
  
  /**
   *  @brief Lock the bitmap sets against changes by other threads
   */
  void lock ();

  /**
   *  @brief Unlock the bitmap sets against changes by other threads
   */
  void unlock ();

  /**
   *  @brief Transfer the content to an QImage 
   */
  void to_image (const std::vector <lay::ViewOp> &view_ops, const lay::DitherPattern &dp, const lay::Drawings *drawings, QImage &img, unsigned int width, unsigned int height);

private:
  void clear_bitmaps ();

  QMutex m_mutex;
  std::vector <lay::Bitmap *> mp_bitmap_buffers;
  std::vector <std::vector <lay::Bitmap *> > mp_drawing_bitmap_buffers;
};

}

#endif

