
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layLayoutView.h"
#include "laySaveLayoutOptionsDialog.h"
#include "dbSaveLayoutOptions.h"
#include "dbStream.h"
#include "tlClassRegistry.h"
#include "layExceptions.h"

namespace lay
{

SaveLayoutOptionsDialog::SaveLayoutOptionsDialog (QWidget *parent, const std::string &title)
  : QDialog (parent, "save_layout_options_dialog"), Ui::SaveLayoutOptionsDialog ()
{
  Ui::SaveLayoutOptionsDialog::setupUi (this);

  setCaption (title.c_str ());

  for (tl::Registrar<db::StreamFormatDeclaration>::iterator fmt = tl::Registrar<db::StreamFormatDeclaration>::begin (); fmt != tl::Registrar<db::StreamFormatDeclaration>::end (); ++fmt) {
    fmt_cbx->addItem (fmt->format_desc ().c_str ());
  }

  connect (buttonBox, SIGNAL (accepted ()), this, SLOT (ok_button_pressed ()));
}

void
SaveLayoutOptionsDialog::ok_button_pressed ()
{
  BEGIN_PROTECTED

  double x = 0.0;
  tl::from_string (dbu_le->text ().ascii (), x);
  tl::from_string (sf_le->text ().ascii (), x);

  accept ();

  END_PROTECTED
}

bool 
SaveLayoutOptionsDialog::get_options (lay::LayoutView *view, unsigned int cv_index, const std::string &fn, bool &gzip, db::SaveLayoutOptions &options)
{
  filename_lbl->setText (fn.c_str ());
  gzip_cb->setChecked (gzip);

  dbu_le->setText (tl::to_string (options.dbu ()).c_str ());

  unsigned int i = 0;
  for (tl::Registrar<db::StreamFormatDeclaration>::iterator fmt = tl::Registrar<db::StreamFormatDeclaration>::begin (); fmt != tl::Registrar<db::StreamFormatDeclaration>::end (); ++fmt, ++i) {
    if (fmt->format_name () == options.format ()) {
      fmt_cbx->setCurrentIndex (i);
      break;
    }
  }

  if (exec ()) {

    int index = fmt_cbx->currentIndex ();
    for (tl::Registrar<db::StreamFormatDeclaration>::iterator fmt = tl::Registrar<db::StreamFormatDeclaration>::begin (); fmt != tl::Registrar<db::StreamFormatDeclaration>::end () && index >= 0; ++fmt, --index) {
      if (index == 0) {
        options.set_format (fmt->format_name ());
      }
    }

    double dbu = 0.0;
    tl::from_string (dbu_le->text ().ascii (), dbu);

    double sf = 1.0;
    tl::from_string (sf_le->text ().ascii (), sf);

    options.set_dbu (dbu);
    options.set_scale_factor (sf);

    if (layersel_cbx->currentIndex () == 0 /*all*/) {
      options.select_all_layers ();
    } else if (layersel_cbx->currentIndex () == 1 /*shown layers*/) {
      for (LayerPropertiesConstIterator layer = view->begin_layers (); layer != view->end_layers (); ++layer) {
        if (layer->cellview_index () == int (cv_index)) {
          options.add_layer (layer->layer_index ());
        }
      }
    } else if (layersel_cbx->currentIndex () == 2 /*visible layers*/) {
      for (LayerPropertiesConstIterator layer = view->begin_layers (); layer != view->end_layers (); ++layer) {
        if (layer->cellview_index () == int (cv_index) && layer->visible (true)) {
          options.add_layer (layer->layer_index ());
        }
      }
    }

    return true;

  } else {
    return false;
  }
}

}

