
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#ifndef HDR_layToScreen.h
#define HDR_layToScreen.h

#include "dbTrans.h"
#include "scPoint.h"

class lay::ToScreen
  : public db::Trans 
{
public:
  ToScreen ();
  ToScreen (const db::Trans &t, sc::Point &s, double mx, double my);

  void transform (const db::Trans &t);
  void swap (db::Trans &t);

  lay::ToScreen &operator*= (const db::Trans &t);
  sc::Point operator* (const db::Point &p) const;

private:
  db::Trans m_t;
  sc::Point m_s;
  double m_mx, m_my;
};

inline void 
lay::ToScreen::transform (const db::Trans &t)
{
  m_t *= t;
}

inline void 
lay::ToScreen::swap (db::Trans &t)
{
  std::swap (t, m_t);
}

inline lay::ToScreen &
lay::ToScreen::operator*= (const db::Trans &t)
{
  transform (t);
  return *this;
}

inline sc::Point 
lay::ToScreen::operator* (const db::Point &p) const
{
  db::Point pp = m_t * p;
  return sc::Point (pp.x () * m_mx + m_s.x (),
                    pp.y () * m_my + m_s.y ());
}

inline lay::ToScreen 
operator* (const db::Trans &t, const lay::ToScreen &s)
{
  lay::ToScreen r (s);
  db::Trans u (t);
  r.swap (u);
  r.transform (u);
  return r;
}

inline lay::ToScreen
operator* (const lay::ToScreen &s, const db::Trans &t)
{
  lay::ToScreen r (s);
  r.transform (t);
  return r;
}

#endif


