
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <QtGui/QGridLayout>
#include <QtGui/QMenu>
#include <QtGui/QColorDialog>
#include <QtGui/QPainter>
#include <QtGui/QPixmap>

#include "layWidgets.h"

#include "tlInternational.h"

namespace lay
{

ColorButton::ColorButton (QWidget *parent, const char *name)
  : QPushButton (parent, name)
{
  QMenu *menu = new QMenu (this);
  menu->addAction (tl::translate ("Automatic").c_str (), this, SLOT (auto_selected ()));
  menu->addAction (tl::translate ("Choose ...").c_str (), this, SLOT (other_selected ()));
  setMenu (menu);
}

ColorButton::ColorButton (QPushButton *&to_replace, const char *name)
  : QPushButton (to_replace->parentWidget (), name)
{
  //  If the push button was part of a layout, replace it.
  //  This is somewhat tricky because there is no common method of
  //  the layout managers to replace a widget.

  QLayout *ly = to_replace->parentWidget ()->layout ();
  if (ly) {

    QBoxLayout *bx_ly = dynamic_cast <QBoxLayout *> (ly);
    if (bx_ly) {
      int i = ly->indexOf (to_replace);
      bx_ly->insertWidget (i, this);
    }

    QGridLayout *grid_ly = dynamic_cast <QGridLayout *> (ly);
    if (grid_ly) {
      int i = ly->indexOf (to_replace);
      int row = 0, column = 0;
      int row_span = 0, column_span = 0;
      grid_ly->getItemPosition (i, &row, &column, &row_span, &column_span);
      grid_ly->addWidget (this, row, column, row_span, column_span);
    }

  }

  delete to_replace;
  to_replace = 0;

  QMenu *menu = new QMenu (this);
  menu->addAction (tl::translate ("Automatic").c_str (), this, SLOT (auto_selected ()));
  menu->addAction (tl::translate ("Choose ...").c_str (), this, SLOT (other_selected ()));
  setMenu (menu);
}

void 
ColorButton::set_color (QColor c)
{
  m_color = c;

  QFontMetrics fm (font (), this);
  QRect rt (fm.boundingRect (tl::translate ("Auto").c_str ()));
  QPixmap pxmp (rt.width () + 24, rt.height () + 8);

  QPainter pxpainter (&pxmp);
  QColor text_color = palette ().color (QPalette::Active, QPalette::Text);
  pxpainter.setPen (QPen (text_color));
  pxpainter.setBrush (QBrush (c.isValid () ? c : QColor (128, 128, 128)));
  QRect r (0, 0, pxmp.width () - 1, pxmp.height () - 1);
  pxpainter.drawRect (r);

  if (! c.isValid ()) {
    pxpainter.setFont (font ());
    pxpainter.setPen (QPen (text_color));
    pxpainter.drawText (r, Qt::AlignHCenter | Qt::AlignVCenter | Qt::TextSingleLine, tl::translate ("Auto").c_str ());
  }

  setIconSize (pxmp.size ());
  setPixmap (pxmp);
}

QColor 
ColorButton::get_color () const
{
  return m_color;
}

void
ColorButton::auto_selected ()
{
  set_color (QColor ());
}

void
ColorButton::other_selected ()
{
  QColor c = QColorDialog::getColor (get_color (), this);
  if (c.isValid ()) {
    set_color (c);
  }
}


}

