
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <tlObserver.h>

namespace tl
{

// --------------------------------------------------------------------
//  Observer implementation

Observer::Observer ()
{ 
  // .. nothing yet ..
}

Observer::Observer (const Observer &d)
{
  operator= (d);
}

Observer &
Observer::operator= (const Observer &d)
{
  if (this != &d) {
    detach_all ();
    for (observed_list_type::const_iterator o = d.m_observed.begin (); o != d.m_observed.end (); ++o) {
      o->first->add_observer (*this);
    }
  }
  return *this;
}

void 
Observer::detach_all ()
{
  while (! m_observed.empty ()) {
    m_observed.front ().first->remove_observer (*this, m_observed.begin ());
  }
}

Observer::~Observer ()
{
  detach_all ();
}

observed_list_type::iterator 
Observer::observer_attach (Observed *observed, observer_list_type::iterator where)
{
  m_observed.push_front (std::make_pair (observed, ListIteratorWrapper (where)));
  return m_observed.begin ();
}

void 
Observer::observer_detach (observed_list_type::iterator where)
{
  m_observed.erase (where);
}

void 
Observer::signal_observed () 
{
  //  base implementation does nothing
}

void 
Observer::signal_observed_int (int) 
{
  //  base implementation does nothing
}

void 
Observer::signal_observed_voidp (void *) 
{
  //  base implementation does nothing
}

unsigned int 
Observer::observes () const 
{
  return m_observed.size ();
}

// --------------------------------------------------------------------
//  Observed implementation

Observed::Observed ()
{
  //  .. nothing yet ..
}

Observed::~Observed ()
{
  remove_observers ();
}

void
Observed::remove_observers ()
{
  while (! m_observers.empty ()) {
    remove_observer (*(m_observers.front ().first));
  }
}

void 
Observed::set_observer (Observer &observer)
{
  remove_observers ();
  add_observer (observer);
}

void 
Observed::add_observer (Observer &observer)
{
  m_observers.push_front (std::make_pair (& observer, ListIteratorWrapper ()));
  m_observers.front ().second = ListIteratorWrapper (observer.observer_attach (this, m_observers.begin ()));
}

void 
Observed::remove_observer (Observer &observer)
{
  for (observed_list_type::iterator o = observer.m_observed.begin (); o != observer.m_observed.end (); ) {
    observed_list_type::iterator oo = o;
    ++oo;
    if (o->first == this) {
      remove_observer (observer, o);
    }
    o = oo;
  }
}

void
Observed::remove_observer (Observer &observer, observed_list_type::iterator where)
{
  m_observers.erase (where->second.iter <observer_list_type::iterator> ());
  observer.observer_detach (where);
}

void 
Observed::signal_observers () const
{
  for (observer_list_type::const_iterator o = m_observers.begin (); o != m_observers.end (); ++o) {
    o->first->signal_observed ();
  }
}

void 
Observed::signal_observers (int arg) const
{
  for (observer_list_type::const_iterator o = m_observers.begin (); o != m_observers.end (); ++o) {
    o->first->signal_observed_int (arg);
  }
}

void 
Observed::signal_observers (void *arg) const
{
  for (observer_list_type::const_iterator o = m_observers.begin (); o != m_observers.end (); ++o) {
    o->first->signal_observed_voidp (arg);
  }
}

unsigned int 
Observed::observers () const 
{
  return m_observers.size ();
}

} // namespace tl


